/* lpack.c -- example program: a simple file packer

   This file is part of the LZO real-time data compression library.

   Copyright (C) 1997 Markus Franz Xaver Johannes Oberhumer

   The LZO library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   The LZO library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the LZO library; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Markus F.X.J. Oberhumer
   markus.oberhumer@jk.uni-linz.ac.at
 */


/*************************************************************************
// NOTE:
//   This is an example program, do not use to compress your data
//
// Please study LZO.FAQ and simple.c first.
**************************************************************************/

#include <lzo1x.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>


static const char *progname = NULL;

static const lzo_uint block_size = (128*1024L);

static unsigned long total_in = 0;
static unsigned long total_out = 0;

/* magic file header */
static const unsigned char magic[] =
	{ 0x00, 0x4c, 0x5a, 0x4f, 0x1a };


/*************************************************************************
// file IO
**************************************************************************/

lzo_uint xread(FILE *f, lzo_voidp buf, lzo_uint len, lzo_bool allow_eof)
{
	lzo_uint l;

	l = fread(buf,1,len,f);
	if (l != len && !allow_eof)
	{
		fprintf(stderr,"\nRead error - premature end of file\n");
		exit(1);
	}
	total_in += l;
	return l;
}


lzo_uint xwrite(FILE *f, const lzo_voidp buf, lzo_uint len)
{
	if (fwrite(buf,1,len,f) != len)
	{
		fprintf(stderr,"\nWrite error  (disk full ?)\n");
		exit(1);
	}
	total_out += len;
	return len;
}


/* read and write portable 32-bit integers */

void xread32(FILE *f, lzo_uint32 *v)
{
	unsigned char b[4];

	xread(f,b,4,0);
	*v  = (lzo_uint32) b[3] <<  0;
	*v |= (lzo_uint32) b[2] <<  8;
	*v |= (lzo_uint32) b[1] << 16;
	*v |= (lzo_uint32) b[0] << 24;
}


void xwrite32(FILE *f, lzo_uint32 v)
{
	unsigned char b[4];

	b[3] = (unsigned char) (v >>  0);
	b[2] = (unsigned char) (v >>  8);
	b[1] = (unsigned char) (v >> 16);
	b[0] = (unsigned char) (v >> 24);
	xwrite(f,b,4);
}


/*************************************************************************
// compress
**************************************************************************/

int do_compress(FILE *fi, FILE *fo)
{
	int r = 0;
	lzo_byte *in = NULL;
	lzo_byte *out = NULL;
	lzo_byte *wrkmem = NULL;
	lzo_uint in_len;
	lzo_uint out_len;
	lzo_uint32 flags = 0;		/* could add some information here */

/*
 * Step 1: allocate compression buffers and work-memory
 */
	in = (lzo_byte *) malloc(block_size);
	out = (lzo_byte *) malloc(block_size + block_size / 64 + 16 + 3);
	wrkmem = (lzo_byte *) malloc(LZO1X_MEM_COMPRESS);
	if (in == NULL || out == NULL || wrkmem == NULL)
	{
		printf("%s: out of memory\n", progname);
		r = 1;
		goto err;
	}

/*
 * Step 2: write magic header & flags
 */
 	xwrite(fo,magic,sizeof(magic));
	xwrite32(fo,flags);

/*
 * Step 3: process blocks
 */
 	for (;;)
	{
		/* read block */
		in_len = xread(fi,in,block_size,1);
		if (in_len <= 0)
			break;

		/* compress block */
		r = lzo1x_1_compress(in,in_len,out,&out_len,wrkmem);
		if (r != LZO_E_OK)
		{
			/* this should NEVER happen */
			printf("internal error - compression failed: %d\n", r);
			r = 2;
			goto err;
		}

		/* write uncompressed block size */
		xwrite32(fo,in_len);

		if (out_len < in_len)
		{
			/* write compressed block */
			xwrite32(fo,out_len);
			xwrite(fo,out,out_len);
		}
		else
		{
			/* not compressible - write uncompressed block */
			xwrite32(fo,in_len);
			xwrite(fo,in,in_len);
		}
	}

	/* write EOF marker */
	xwrite32(fo,0);

	r = 0;
err:
	if (wrkmem) free(wrkmem);
	if (out) free(out);
	if (in) free(in);
	return r;
}


/*************************************************************************
// decompress
//
// possible improvement: we could use overlapping decompression to
//   save some memory - see overlap.c
**************************************************************************/

int do_decompress(FILE *fi, FILE *fo)
{
	int r = 0;
	lzo_byte *in = NULL;
	lzo_byte *out = NULL;
	lzo_uint in_len;
	lzo_uint out_len;
	unsigned char m [ sizeof(magic) ];
	lzo_uint32 flags;

/*
 * Step 1: allocate compression buffers
 */
	in = (lzo_byte *) malloc(block_size);
	out = (lzo_byte *) malloc(block_size);
	if (in == NULL || out == NULL)
	{
		if (out) free(out);
		if (in) free(in);
		printf("%s: out of memory\n", progname);
		r = 1;
		goto err;
	}

/*
 * Step 2: check magic header
 */
 	if (xread(fi,m,sizeof(magic),1) != sizeof(magic) ||
	    memcmp(m,magic,sizeof(magic)) != 0)
    {
		printf("%s: header error - this file is not compressed by lpack\n", progname);
		r = 2;
		goto err;
	}

	xread32(fi,&flags);

/*
 * Step 3: process blocks
 */
 	for (;;)
	{
		/* read uncompressed size */
		xread32(fi,&out_len);

		/* exit if last block */
		if (out_len == 0)
			break;

		/* read compressed size */
		xread32(fi,&in_len);

		if (in_len > out_len || in_len > block_size || out_len > block_size)
		{
			printf("%s: data corrupted\n", progname);
			r = 3;
			goto err;
		}

		/* read compressed block */
		xread(fi,in,in_len,0);

		if (in_len < out_len)
		{
			/* decompress (use safe decompressor as data may be corrupted) */
			lzo_uint d = out_len;
			r = lzo1x_decompress_safe(in,in_len,out,&d,NULL);
			if (r != LZO_E_OK || d != out_len)
			{
				printf("%s: compressed data violation\n", progname);
				r = 4;
				goto err;
			}
			/* write uncompressed block */
			xwrite(fo,out,out_len);
		}
		else
		{
			/* write original block */
			xwrite(fo,in,in_len);
		}
	}

	r = 0;
err:
	if (out) free(out);
	if (in) free(in);
	return r;
}


/*************************************************************************
//
**************************************************************************/

int main(int argc, char *argv[])
{
	int i = 1;
	int r = 0;
	FILE *fi;
	FILE *fo;
	const char *in_name = 0;
	const char *out_name = 0;
	lzo_bool opt_decompress = 0;

#if defined(__EMX__)
	_response(&argc,&argv);
	_wildcard(&argc,&argv);
#endif

	printf("\nLZO real-time data compression library (v%s, %s).\n",
	        LZO_VERSION_STRING, LZO_VERSION_DATE);
	printf("Copyright (C) 1996, 1997 Markus Franz Xaver Johannes Oberhumer\n\n");

	printf(
"*** DISCLAIMER ***\n"
"   This is an example program, do not use to compress your data !\n"
"   Get LZOP if you're interested into a full-featured LZO packer.\n"
"   See http://www.infosys.tuwien.ac.at/Staff/lux/marco/lzop.html\n"
"\n");

	progname = argv[0];
	if (i < argc && strcmp(argv[i],"-d") == 0)
		opt_decompress = 1, i++;
	if (i + 2 != argc)
	{
		printf("usage: %s [-d] input-file output-file\n", progname);
		exit(1);
	}
 	in_name = argv[i++];
 	out_name = argv[i++];

/*
 * Step 1: initialize the LZO library
 */
	if (lzo_init() != LZO_E_OK)
	{
		printf("lzo_init() failed !!!\n");
		exit(1);
	}

/*
 * Step 2: open the files
 */
	fi = fopen(in_name,"rb");
	if (fi == NULL)
	{
		printf("%s: cannot open input file %s\n", progname, in_name);
		exit(1);
	}

	/* this is an example program, so make sure we don't overwrite a file */
	fo = fopen(out_name,"rb");
	if (fo != NULL)
	{
		printf("%s: file %s already exists -- not overwritten\n", progname, out_name);
		fclose(fi);
		fclose(fo);
		exit(1);
	}
	fo = fopen(out_name,"wb");
	if (fo == NULL)
	{
		printf("%s: cannot open output file %s\n", progname, out_name);
		fclose(fi);
		exit(1);
	}

/*
 * Step 3: process file
 */
 	if (opt_decompress)
	{
		r = do_decompress(fi,fo);
		if (r == 0)
			printf("%s: decompressed %ld bytes into %ld\n",
			        progname, total_in, total_out);
	}
	else
	{
		r = do_compress(fi,fo);
		if (r == 0)
			printf("%s: compressed %ld bytes into %ld\n",
			        progname, total_in, total_out);
	}

	fclose(fi);
	fclose(fo);
	return r;
}

/*
vi:ts=4
*/

