/* overlap.c -- example program: overlapping (in-place) decompression

   This file is part of the LZO real-time data compression library.

   Copyright (C) 1997 Markus Franz Xaver Johannes Oberhumer
   Copyright (C) 1996 Markus Franz Xaver Johannes Oberhumer

   The LZO library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   The LZO library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with the LZO library; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Markus F.X.J. Oberhumer
   markus.oberhumer@jk.uni-linz.ac.at
 */


/*************************************************************************
// This program shows how to do overlapping (in-place) decompression.
//
// Please study LZO.FAQ and simple.c first.
**************************************************************************/

#include <lzo1x.h>

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>


static const char *progname = NULL;

#define HEAP_ALLOC(var,size) \
	long __LZO_MMODEL var [ ((size) + (sizeof(long) - 1)) / sizeof(long) ]
static HEAP_ALLOC(wrkmem,LZO1X_MEM_COMPRESS);


/* Overhead (in bytes) for the in-place decompression buffer.
 * 1024 should be really more than enough - most files need only 16 !
 *
 * Worst case (for files that are compressible by only a few bytes)
 * is 'in_len + in_len / 64 + 16 + 3'.
 */
static lzo_uint overlap_overhead = 1024;


/*************************************************************************
//
**************************************************************************/

/* for test purposes only */
LZO_EXTERN(int)
lzo1x_decompress_asm_fast ( const lzo_byte *src, lzo_uint  src_len,
                                  lzo_byte *dst, lzo_uint *dst_len,
                                  lzo_voidp wrkmem /* NOT USED */ );



int do_file ( const char *in_name )
{
	int r;
	FILE *f;

	lzo_byte *in = NULL;
	lzo_uint in_len;

	lzo_byte *out = NULL;
	lzo_uint out_len;

	lzo_byte *overlap = NULL;
	lzo_uint overlap_len;
	lzo_uint overlap_offset;

	lzo_uint new_len = 0;

/*
 * Step 1: open the input file
 */
 	f = fopen(in_name,"rb");
	if (f == 0)
	{
		printf("%s: cannot open file %s\n", progname, in_name);
		return 0;
	}
	fseek(f,0,SEEK_END);
	in_len = ftell(f);
	fseek(f,0,SEEK_SET);

/*
 * Step 2: allocate compression buffers and read the file
 */
 	out_len = in_len + in_len / 64 + 16 + 3;
	in = (lzo_byte *) malloc(in_len);
	out = (lzo_byte *) malloc(out_len);
	if (in == 0 || out == 0)
	{
		printf("%s: out of memory\n", progname);
		exit(1);
	}
	in_len = fread(in,1,in_len,f);
	fclose(f);
	printf("%s: loaded file %s: %ld bytes\n", progname, in_name, (long) in_len);

/*
 * Step 3: compress from `in' to `out' with LZO1X-1
 */
	r = lzo1x_1_compress(in,in_len,out,&out_len,wrkmem);
	if (r != LZO_E_OK)
	{
		/* this should NEVER happen */
		printf("internal error - compression failed: %d\n", r);
		exit(1);
	}
	printf("%26s %8lu -> %8lu\n", "LZO1X-1:", (long) in_len, (long) out_len);

/*
 * Step 4: check if compressible
 */
 	if (out_len >= in_len)
	{
		printf("This file contains incompressible data.\n");
		goto next_file;
	}

/*
 * Step 5: allocate the `overlap' buffer
 */
	overlap_len = in_len + overlap_overhead;
	overlap = (lzo_byte *) malloc(overlap_len);
	if (overlap == 0)
	{
		printf("%s: out of memory\n", progname);
		exit(1);
	}

/*
 * Step 6: prepare data in `overlap' buffer.
 *         copy compressed data at the top of the overlap buffer
 */
	overlap_offset = overlap_len - out_len;
	lzo_memcpy(overlap + overlap_offset, out, out_len);

/*
 * Step 7: do an in-place decompression within the `overlap' buffer
 */
	new_len = in_len;
#if 1
	r = lzo1x_decompress(overlap+overlap_offset,out_len,overlap,&new_len,NULL);
#else
	r = lzo1x_decompress_asm_fast(overlap+overlap_offset,out_len,overlap,&new_len,NULL);
#endif
	if (r != LZO_E_OK)
	{
		/* this may happen if overhead is too small */
		printf("overlapping decompression failed: %d - increase `overlap_overhead'\n", r);
		exit(1);
	}

/*
 * Step 8: verify decompression
 */
 	if (new_len != in_len || lzo_memcmp(in,overlap,in_len) != 0)
	{
		/* this may happen if overhead is too small */
		printf("overlapping decompression data error - increase `overlap_overhead'\n");
		exit(1);
	}


	printf("overlapping decompression: %8lu -> %8lu    overhead: %5ld\n",
			(long) out_len, (long) new_len, (long) overlap_overhead);

next_file:
	if (overlap) free(overlap);
	if (out) free(out);
	if (in) free(in);

	return 0;
}


/*************************************************************************
//
**************************************************************************/

int main(int argc, char *argv[])
{
	int r;
	int i = 1;

#if defined(__EMX__)
	_response(&argc,&argv);
	_wildcard(&argc,&argv);
#endif

	printf("\nLZO real-time data compression library (v%s, %s).\n",
	        LZO_VERSION_STRING, LZO_VERSION_DATE);
	printf("Copyright (C) 1996, 1997 Markus Franz Xaver Johannes Oberhumer\n\n");

	progname = argv[0];
	if (i < argc && argv[i][0] == '-')
		overlap_overhead = atoi(&argv[i++][1]);
	if (overlap_overhead < 8)
	{
		printf("%s: invalid overhead value %d\n", progname, overlap_overhead);
		exit(1);
	}
	if (i >= argc)
	{
		printf("usage: %s [-overhead_in_bytes] file..\n", progname);
		exit(1);
	}

/*
 * Step 1: initialize the LZO library
 */
	if (lzo_init() != LZO_E_OK)
	{
		printf("lzo_init() failed !!!\n");
		exit(1);
	}

/*
 * Step 2: process files
 */
	for (r = 0; r == 0 && i < argc; i++)
		r = do_file(argv[i]);

	return r;
}

/*
vi:ts=4
*/

